#############################################################################
##
#W    posets.gd        share package 'homology'                 Welker
##
## Some example posets (in particular from group theory), constructors for 
## posets and their order complexes.

#==================================================
# Examples of Order-Relations
#==================================================


#############################################################################
##
#F  IsSetPartitionRefinement( <p1> , <p2> )
##
##  returns true if the set-partition `<p2>' is a refinement of the 
##  set-partition `<p1>' and false otherwise.
##
DeclareGlobalFunction ("IsSetPartitionRefinement");

#############################################################################
##
#F  IsNumberPartitionRefinement ( <p1> , <p2> )
##
##  returns true if the integer-partition `<p2>' is a refinement of the 
##  integer-partition `<p1>' and false otherwise.
##
DeclareGlobalFunction ("IsNumberPartitionRefinement");

#############################################################################
##
#F  IsCompositionPartitionRefinement ( <c1> , <c2> )
##
##  returns true if the composition `<c22>' is a refinement of the 
##  integer-partition `<c1>' and false otherwise.
##
DeclareGlobalFunction ("IsCompositionRefinement");



#=======================================================
# Examples of Groundsets for the above Order Relations
#=======================================================

#############################################################################
##
#F  Subsets ( <s> )
##
##  returns a list of all subsets of the set `<s>'
##
DeclareGlobalFunction ("Subsets");

#############################################################################
##
#F  NonTrivialSubsets ( <s> )
##
##  returns a list of all non-trivial subsets of the set `<s>'
##
DeclareGlobalFunction ("NonTrivialSubsets");

#############################################################################
##
#F  NonTrivialPartitionsSet ( <s> )
##
##  returns a list of all non-trivial partitions of the set `<s>'
##
DeclareGlobalFunction ("NonTrivialPartitionsSet");

#############################################################################
##
#F  PartitionsNumber ( <n> )
##
##  returns a list of all number-partitions of the number `<n>'
##
DeclareGlobalFunction ("PartitionsNumber");

#############################################################################
##
#F  NonTrivialPartitionsNumber ( <n> )
##
##  returns a list of all non-trivial number-partitions of the number `<n>'
##
DeclareGlobalFunction ("NonTrivialPartitionsNumber");

#############################################################################
##
#F  Compositions ( <n> )
##
##  returns a list of all composition of the number `<n>'
##  Note: Gap calls a composition ``OrderedPartition'' 
##
DeclareGlobalFunction ("Compositions");

#############################################################################
##
#F  NonTrivialCompositions ( <n> )
##
##  returns a list of all non-trivial composition of the number `<n>'
##
DeclareGlobalFunction ("NonTrivialCompositions");

#############################################################################
##
#F  SubSemilatticeBoolean ( <list_of_subsets> )
##
##  returns the subsemi-lattice of the Boolean lattice generated by
##  the sets in the list <list_of_subsets>
##
DeclareGlobalFunction ("SubSemilatticeBoolean");




#==================================================
# Constructors for Posets and Order Complexes
#==================================================

#############################################################################
##
#F  OrderRelationToPoset ( <s> , <rel> )
##
##  constructs the poset in form of the Hasse diagram corresponding to the
##  order relation `<rel>' which is a binary boolean function on the ground
##  set `<s>' 
##
DeclareGlobalFunction("OrderRelationToPoset");

#############################################################################
##
#F  OrderComplex ( <p> , <min, max )
##
##  constructs the order complex of open interval (`<min>',`<max>') in 
##  the poset `<p>' 
##
DeclareGlobalFunction("IntervalOrderComplex");

#############################################################################
##
#F  OrderComplex ( <p> )
##
##  constructs the order complex of the poset `<p>' 
##
DeclareGlobalFunction ("OrderComplex");

