#############################################################################
##
#W    simplicialhomology.gi   share package 'homology'      Dumas
##
##==========================================================
## Smiht form routines
## (c) July 1999
##==========================================================

HomologyInfo (50); # default info level

#==================================================
# Conversions GAP dense matrix & sparse matrix
#==================================================
InstallGlobalFunction (gap2SM, function (mat, filename)
  local ni, nj, i, j, output;
  if (not IsMatrix(mat)) then
    Info (InfoWarning, 1, "Argument must be a matrix.");
    return fail;
  fi;
# in documentation : IsWriteableFile
# in practice      : IsWritableFile
# So no test for now ...
#  if (not IsWritableFile(filename)) then
#    Info (InfoWarning, 1, "Second argument must be a writeable filename.");
#    return fail;
#  fi;
  ni := Length(mat); nj := Length(mat[1]);
  output := OutputTextFile( filename, false );;
  AppendTo(output,ni," ",nj," M\n");
  for i in [1..ni] do
    for j in [1..nj] do
      if (mat[i][j] <> 0) then
	AppendTo(output,i," ",j," ",mat[i][j],"\n");
      fi;
    od;
  od;
  AppendTo(output,"0 0 0\n");
  CloseStream(output);
end);

#==================================================
# Local argument handler for smith_form executable
#==================================================
ComputeMatrixRanks := function (args, procname, mode, coeff)
  local nargs, mat, AlgStr, Algorithm, tmpdir, streamin, streamout,
        arguments, fname, s, Status, Usage, ErrorTolerance, ETStr;

  Usage:= function ()
    Print ("usage: ", procname, " (<matrix> | <filename>");
    if (mode >= 4  or mode = 2) then Print (", <modulus>)   "); else Print(")   "); fi;
    Print ("where:\n");
    Print ("matrix    : a gap matrix\n");
    Print ("filename  : a filename containing a Sparse Matrix\n");
    if (mode >= 4 or mode = 2) then
      Print ("modulus : the coefficient of the torsion ranks wanted\n");
    fi;
      Print ("The Option \"HomologyAlgorithm\" can select the algorithm to be used:\n");
      Print ("             - \"Valence-elim\" : ranks via elimination\n");
      Print ("             - \"Valence-bb\"   : ranks via BlackBox methods\n");
      Print ("The Option \"UncertaintyTolerance\" can set the error tolerance for the Valence algorithms:\n");
      Print ("  - 0                     : The algorithm chooses some tolerance (default)\n");
      Print ("  - 1                     : The algorithm chooses some tolerance\n");
      Print ("  - a Rational r, 0<r<1   : Takes r as error tolerance\n");
      Print ("  - a Rational r, r>1     : Takes 1/r as error tolerance\n");
      Print ("  - an Integer i, i!=0    : Takes 1/i as error tolerance\n");
  end;

  nargs := Length (args);
  if nargs < 1 then
    Info (InfoWarning, 1, "Error : Invalid number of arguments\n");
    Usage ();
    return fail;
  fi;
  if mode = 2 and (not (coeff in Integers) or (coeff < 2)) then
    Info (InfoWarning, 1, "Modulus must be an integer >= 2.");
    return fail;
  fi;

  tmpdir := DirectoryTemporary ();
  mat := args [1];
  if (IsMatrix(mat)) then
    fname := Filename (tmpdir, "SM_tmp");
    gap2SM(mat,fname);
  else
    fname := mat;
  fi;
  
  if (not IsReadableFile(fname)) then
    Info (InfoWarning, 1, "First argument must be a matrix or a readable filename.");
    return fail;
  fi;

  ErrorTolerance := 0;
  ETStr := ValueOption("UncertaintyTolerance");
  if ETStr <> fail then
    ErrorTolerance := ETStr;
    if ErrorTolerance = fail then
        ErrorTolerance := 0;
    fi;
  fi;

  Algorithm := 3;
  AlgStr := ValueOption ("HomologyAlgorithm");
  if AlgStr <> fail then
    Algorithm := First ([1 .. Length (HomologyAlgorithms)], i -> HomologyAlgorithms [i] = AlgStr);
    if Algorithm = fail then
      Algorithm := 3;
      Info (InfoWarning, 2, "Invalid homology algorithm set! Using the default.");
    fi;
  fi;

  arguments := [String (fname), String (InfoLevel (InfoHomology)), String (InfoLevel (InfoTiming)), String (Algorithm), String (mode), String (coeff),  String(NumeratorRat(ErrorTolerance)), String(DenominatorRat(ErrorTolerance))];
  fname := Filename (tmpdir, "SNFI_tmp");
  streamout := OutputTextFile (fname, false);
  CloseStream (streamout);
  streamin := InputTextFile (fname);
  s := "return ";
  streamout := OutputTextString (s, true);
  Status := Process (DirectoryCurrent (), Filename (DirectoriesPackagePrograms ("homology"), "smithform_gap"), streamin, streamout, arguments);
  WriteLine (streamout, ";");
  CloseStream (streamout);
  CloseStream (streamin);
  if Status <> 0 then
    return fail;
  else
    return ReadAsFunction (InputTextString (s)) ();
  fi;
end;

#==================================================
# Computing information sparse matrices
#==================================================

InstallGlobalFunction (SMSmithForm, function (arg)
  local l, res, ll, inv, i, j, n;
  if Length (arg) < 2 then
    l := ComputeMatrixRanks (arg, "SMSmithForm", 0, 0); 
  else
    l := ComputeMatrixRanks (arg, "SMModularSmithForm", 2, arg [2] );
  fi;

  if (l[1]>l[2]) then
	n := l[2];
  else
	n := l[1];
  fi;
  res := [l[1],l[2]];
  ll := [];
  for i in [3..Length(l)] do
    inv := l[i];
    for j in [1..inv[2]] do
	Add(ll, inv[1]);
    od;
  od;
  for i in [Length(ll)+1..n] do
	Add(ll,0);
  od;
  Add(res, ll);
  return res;
end);

InstallGlobalFunction (SMInvariantFactors, function (arg)
  local l, n, inv, i;
  if Length (arg) < 2 then
  	l:= ComputeMatrixRanks (arg, "SMInvariantFactors", 0, 0);
  else
  	l:= ComputeMatrixRanks (arg, "SMModularSmithForm", 2, arg [2] );
  fi;
  if (l[1]>l[2]) then
        n := l[2];
  else
        n := l[1];
  fi;
  for i in [3..Length(l)] do
    inv := l[i];
    n := n - inv[2];
  od;
  Add(l, [0, n]);
  return l;
end);

InstallGlobalFunction (SMIntegerRank, function (arg)
  local l;
  l:= ComputeMatrixRanks (arg, "SMIntegerRank", 1, 0);
  return l[3][2];
end);

InstallGlobalFunction (SMPrimeRank, function (arg)
  local Temp, p, l;
  if Length (arg) < 2 then
    Info (InfoWarning, 1, "A prime must be specified for prime rank computation.");
    return fail;
  else
    p := arg [2];
    arg := List (Concatenation ([1], [3 .. Length (arg)]), i -> arg [i]);
  fi;
  l:= ComputeMatrixRanks(arg, "SMPrimeRank", 4, p);
  return l[3][2];
end);

InstallGlobalFunction (SMPrimePowerRank, function (arg)
  local Temp, p, l;
  if Length (arg) < 2 then
    Info (InfoWarning, 1, "A prime power must be specified for prime rank computation.");
    return fail;
  else
    p := arg [2];
    arg := List (Concatenation ([1], [3 .. Length (arg)]), i -> arg [i]);
  fi;
  l:= ComputeMatrixRanks(arg, "SMPrimePowerRank", 5, p);
  return l[3][2];
end);
