// =================================================================== //
// Givaro : RSA scheme.
// Time-stamp: <21 Mar 00 15:58:14 Jean-Guillaume.Dumas@imag.fr> 
// =================================================================== //

#ifndef _GIVARO_RSA_
#define _GIVARO_RSA_

#include <iostream>
#include "givinteger.h"
#include "givintprime.h"
#include "givintfactor.h"
#include "givrandom.h"

// =================================================================== //
// RSA public-key cipher codes
// =================================================================== //

template<class RandIter = GivRandom>
class IntRSADom : public IntFactorDom<RandIter> {
public:
    // JGD 19.02.2003 : Should work nicely, but produces:
    //                  "implicit typename is deprecated"
    // using IntFactorDom<RandIter>::element;
    // using IntFactorDom<RandIter>::random_generator;
    typedef typename IntFactorDom<RandIter>::element element;
    typedef typename IntFactorDom<RandIter>::random_generator random_generator;

private:
    element _m, _k;
    element _u;
    long _lm;

public:

    IntRSADom(RandIter& g = *(new RandIter()) ) : IntFactorDom<RandIter>(g) { keys_gen(_g, 100, 90, _m, _k, _u); _lm = log(m,10); }
    IntRSADom(const long p, const long q, RandIter& g = *(new RandIter()) ) : IntFactorDom<RandIter>(g) { keys_gen(_g, p, q, _m, _k, _u); _lm = log(_m,10); }
    IntRSADom(const element& m, const element& k, const element& u) : _m(m), _k(k), _u(u), _lm(log(m,10)) {}
    IntRSADom(const element& m, const element& k) : _m(m), _k(k), _u(0), _lm(log(m,10)) {}

// =================================================================== //
// Accesses
// =================================================================== //
    const element& getm() const { return _m; }
    const element& getk() const { return _k; }
    const element& getu() const { return _u; }

// =================================================================== //
// Text conversions
// =================================================================== // 
    ostream& encipher(ostream&, istream&) const ;
    ostream& decipher(ostream&, istream&) ;

protected:
// =================================================================== //
// Keys generation
// public keys are m and k, the secret key is u.
// ciphering is computing       : x^k mod m
// deciphering is computing     : b^u mod m
// since for any x, x^(k.u) = x mod m
// =================================================================== //

// =================================================================== //
// Here m = p*q
// p and q are prime numbers of respective sizes psize, qsize
// Moreover p-1 and q-1 have one prime factor of respective size 2/3
// since k.u = 1 mod (p-1)(q-1)
// =================================================================== //
    void keys_gen(random_generator& g, long psize, long qsize, element& m, element& k, element& u) const ;
    
// =================================================================== //
// log[10]
// =================================================================== //
    long log(const element& n, const long) const ;

// =================================================================== //
// Text conversions
// =================================================================== // 
    ostream& ecriture_str(ostream&, const element&) const ;
    ostream& ecriture_Int(ostream&, const element&, const long) const ;

public:
// =================================================================== //
// Breaking codes : finding u knowing only m an k ...
// =================================================================== //
    element& point_break(element& u) ;

};

#include "givintrsa.inl"

#endif
